@@ -8,9 +8,10 @@ import (
 	"github.com/go-errors/errors"
 	"github.com/jesseduffield/lazygit/pkg/commands/models"
 	"github.com/jesseduffield/lazygit/pkg/commands/oscommands"
-	"github.com/jesseduffield/lazygit/pkg/commands/types/enums"
+	"github.com/jesseduffield/lazygit/pkg/common"
 	"github.com/jesseduffield/lazygit/pkg/config"
 	"github.com/jesseduffield/lazygit/pkg/utils"
+	"github.com/samber/lo"
 	"github.com/stefanhaller/git-todo-parser/todo"
 	"github.com/stretchr/testify/assert"
 )
@@ -28,52 +29,48 @@ var singleCommitOutput = strings.Replace(`0eea75e8c631fba6b58135697835d58ba4c18d
 
 func TestGetCommits(t *testing.T) {
 	type scenario struct {
-		testName        string
-		runner          *oscommands.FakeCmdObjRunner
-		expectedCommits []*models.Commit
-		expectedError   error
-		logOrder        string
-		rebaseMode      enums.RebaseMode
-		opts            GetCommitsOptions
-		mainBranches    []string
+		testName           string
+		runner             *oscommands.FakeCmdObjRunner
+		expectedCommitOpts []models.NewCommitOpts
+		expectedError      error
+		logOrder           string
+		opts               GetCommitsOptions
+		mainBranches       []string
 	}
 
 	scenarios := []scenario{
 		{
-			testName:   "should return no commits if there are none",
-			logOrder:   "topo-order",
-			rebaseMode: enums.REBASE_MODE_NONE,
-			opts:       GetCommitsOptions{RefName: "HEAD", RefForPushedStatus: "mybranch", IncludeRebaseCommits: false},
+			testName: "should return no commits if there are none",
+			logOrder: "topo-order",
+			opts:     GetCommitsOptions{RefName: "HEAD", RefForPushedStatus: "mybranch", IncludeRebaseCommits: false},
 			runner: oscommands.NewFakeRunner(t).
 				ExpectGitArgs([]string{"merge-base", "mybranch", "mybranch@{u}"}, "b21997d6b4cbdf84b149d8e6a2c4d06a8e9ec164", nil).
-				ExpectGitArgs([]string{"log", "HEAD", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, "", nil),
+				ExpectGitArgs([]string{"log", "HEAD", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%P%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, "", nil),
 
-			expectedCommits: []*models.Commit{},
-			expectedError:   nil,
+			expectedCommitOpts: []models.NewCommitOpts{},
+			expectedError:      nil,
 		},
 		{
-			testName:   "should use proper upstream name for branch",
-			logOrder:   "topo-order",
-			rebaseMode: enums.REBASE_MODE_NONE,
-			opts:       GetCommitsOptions{RefName: "refs/heads/mybranch", RefForPushedStatus: "refs/heads/mybranch", IncludeRebaseCommits: false},
+			testName: "should use proper upstream name for branch",
+			logOrder: "topo-order",
+			opts:     GetCommitsOptions{RefName: "refs/heads/mybranch", RefForPushedStatus: "refs/heads/mybranch", IncludeRebaseCommits: false},
 			runner: oscommands.NewFakeRunner(t).
 				ExpectGitArgs([]string{"merge-base", "refs/heads/mybranch", "mybranch@{u}"}, "b21997d6b4cbdf84b149d8e6a2c4d06a8e9ec164", nil).
-				ExpectGitArgs([]string{"log", "refs/heads/mybranch", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, "", nil),
+				ExpectGitArgs([]string{"log", "refs/heads/mybranch", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%P%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, "", nil),
 
-			expectedCommits: []*models.Commit{},
-			expectedError:   nil,
+			expectedCommitOpts: []models.NewCommitOpts{},
+			expectedError:      nil,
 		},
 		{
 			testName:     "should return commits if they are present",
 			logOrder:     "topo-order",
-			rebaseMode:   enums.REBASE_MODE_NONE,
 			opts:         GetCommitsOptions{RefName: "HEAD", RefForPushedStatus: "mybranch", IncludeRebaseCommits: false},
 			mainBranches: []string{"master", "main", "develop"},
 			runner: oscommands.NewFakeRunner(t).
 				// here it's seeing which commits are yet to be pushed
 				ExpectGitArgs([]string{"merge-base", "mybranch", "mybranch@{u}"}, "b21997d6b4cbdf84b149d8e6a2c4d06a8e9ec164", nil).
 				// here it's actually getting all the commits in a formatted form, one per line
-				ExpectGitArgs([]string{"log", "HEAD", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, commitsOutput, nil).
+				ExpectGitArgs([]string{"log", "HEAD", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%P%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, commitsOutput, nil).
 				// here it's testing which of the configured main branches have an upstream
 				ExpectGitArgs([]string{"rev-parse", "--symbolic-full-name", "master@{u}"}, "refs/remotes/origin/master", nil).       // this one does
 				ExpectGitArgs([]string{"rev-parse", "--symbolic-full-name", "main@{u}"}, "", errors.New("error")).                   // this one doesn't, so it checks origin instead
@@ -84,7 +81,7 @@ func TestGetCommits(t *testing.T) {
 				// here it's seeing where our branch diverged from the master branch so that we can mark that commit and parent commits as 'merged'
 				ExpectGitArgs([]string{"merge-base", "HEAD", "refs/remotes/origin/master", "refs/remotes/origin/main"}, "26c07b1ab33860a1a7591a0638f9925ccf497ffa", nil),
 
-			expectedCommits: []*models.Commit{
+			expectedCommitOpts: []models.NewCommitOpts{
 				{
 					Hash:          "0eea75e8c631fba6b58135697835d58ba4c18dbc",
 					Name:          "better typing for rebase mode",
@@ -203,14 +200,13 @@ func TestGetCommits(t *testing.T) {
 		{
 			testName:     "should not call merge-base for mainBranches if none exist",
 			logOrder:     "topo-order",
-			rebaseMode:   enums.REBASE_MODE_NONE,
 			opts:         GetCommitsOptions{RefName: "HEAD", RefForPushedStatus: "mybranch", IncludeRebaseCommits: false},
 			mainBranches: []string{"master", "main"},
 			runner: oscommands.NewFakeRunner(t).
 				// here it's seeing which commits are yet to be pushed
 				ExpectGitArgs([]string{"merge-base", "mybranch", "mybranch@{u}"}, "b21997d6b4cbdf84b149d8e6a2c4d06a8e9ec164", nil).
 				// here it's actually getting all the commits in a formatted form, one per line
-				ExpectGitArgs([]string{"log", "HEAD", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, singleCommitOutput, nil).
+				ExpectGitArgs([]string{"log", "HEAD", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%P%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, singleCommitOutput, nil).
 				// here it's testing which of the configured main branches exist; neither does
 				ExpectGitArgs([]string{"rev-parse", "--symbolic-full-name", "master@{u}"}, "", errors.New("error")).
 				ExpectGitArgs([]string{"rev-parse", "--verify", "--quiet", "refs/remotes/origin/master"}, "", errors.New("error")).
@@ -219,7 +215,7 @@ func TestGetCommits(t *testing.T) {
 				ExpectGitArgs([]string{"rev-parse", "--verify", "--quiet", "refs/remotes/origin/main"}, "", errors.New("error")).
 				ExpectGitArgs([]string{"rev-parse", "--verify", "--quiet", "refs/heads/main"}, "", errors.New("error")),
 
-			expectedCommits: []*models.Commit{
+			expectedCommitOpts: []models.NewCommitOpts{
 				{
 					Hash:          "0eea75e8c631fba6b58135697835d58ba4c18dbc",
 					Name:          "better typing for rebase mode",
@@ -240,14 +236,13 @@ func TestGetCommits(t *testing.T) {
 		{
 			testName:     "should call merge-base for all main branches that exist",
 			logOrder:     "topo-order",
-			rebaseMode:   enums.REBASE_MODE_NONE,
 			opts:         GetCommitsOptions{RefName: "HEAD", RefForPushedStatus: "mybranch", IncludeRebaseCommits: false},
 			mainBranches: []string{"master", "main", "develop", "1.0-hotfixes"},
 			runner: oscommands.NewFakeRunner(t).
 				// here it's seeing which commits are yet to be pushed
 				ExpectGitArgs([]string{"merge-base", "mybranch", "mybranch@{u}"}, "b21997d6b4cbdf84b149d8e6a2c4d06a8e9ec164", nil).
 				// here it's actually getting all the commits in a formatted form, one per line
-				ExpectGitArgs([]string{"log", "HEAD", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, singleCommitOutput, nil).
+				ExpectGitArgs([]string{"log", "HEAD", "--topo-order", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%P%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, singleCommitOutput, nil).
 				// here it's testing which of the configured main branches exist
 				ExpectGitArgs([]string{"rev-parse", "--symbolic-full-name", "master@{u}"}, "refs/remotes/origin/master", nil).
 				ExpectGitArgs([]string{"rev-parse", "--symbolic-full-name", "main@{u}"}, "", errors.New("error")).
@@ -258,7 +253,7 @@ func TestGetCommits(t *testing.T) {
 				// here it's seeing where our branch diverged from the master branch so that we can mark that commit and parent commits as 'merged'
 				ExpectGitArgs([]string{"merge-base", "HEAD", "refs/remotes/origin/master", "refs/remotes/origin/develop", "refs/remotes/origin/1.0-hotfixes"}, "26c07b1ab33860a1a7591a0638f9925ccf497ffa", nil),
 
-			expectedCommits: []*models.Commit{
+			expectedCommitOpts: []models.NewCommitOpts{
 				{
 					Hash:          "0eea75e8c631fba6b58135697835d58ba4c18dbc",
 					Name:          "better typing for rebase mode",
@@ -277,43 +272,41 @@ func TestGetCommits(t *testing.T) {
 			expectedError: nil,
 		},
 		{
-			testName:   "should not specify order if `log.order` is `default`",
-			logOrder:   "default",
-			rebaseMode: enums.REBASE_MODE_NONE,
-			opts:       GetCommitsOptions{RefName: "HEAD", RefForPushedStatus: "mybranch", IncludeRebaseCommits: false},
+			testName: "should not specify order if `log.order` is `default`",
+			logOrder: "default",
+			opts:     GetCommitsOptions{RefName: "HEAD", RefForPushedStatus: "mybranch", IncludeRebaseCommits: false},
 			runner: oscommands.NewFakeRunner(t).
 				ExpectGitArgs([]string{"merge-base", "mybranch", "mybranch@{u}"}, "b21997d6b4cbdf84b149d8e6a2c4d06a8e9ec164", nil).
-				ExpectGitArgs([]string{"log", "HEAD", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, "", nil),
+				ExpectGitArgs([]string{"log", "HEAD", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%P%x00%m%x00%s", "--abbrev=40", "--no-show-signature", "--"}, "", nil),
 
-			expectedCommits: []*models.Commit{},
-			expectedError:   nil,
+			expectedCommitOpts: []models.NewCommitOpts{},
+			expectedError:      nil,
 		},
 		{
-			testName:   "should set filter path",
-			logOrder:   "default",
-			rebaseMode: enums.REBASE_MODE_NONE,
-			opts:       GetCommitsOptions{RefName: "HEAD", RefForPushedStatus: "mybranch", FilterPath: "src"},
+			testName: "should set filter path",
+			logOrder: "default",
+			opts:     GetCommitsOptions{RefName: "HEAD", RefForPushedStatus: "mybranch", FilterPath: "src"},
 			runner: oscommands.NewFakeRunner(t).
 				ExpectGitArgs([]string{"merge-base", "mybranch", "mybranch@{u}"}, "b21997d6b4cbdf84b149d8e6a2c4d06a8e9ec164", nil).
-				ExpectGitArgs([]string{"log", "HEAD", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%m%x00%s", "--abbrev=40", "--follow", "--no-show-signature", "--", "src"}, "", nil),
+				ExpectGitArgs([]string{"log", "HEAD", "--oneline", "--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%P%x00%m%x00%s", "--abbrev=40", "--follow", "--no-show-signature", "--", "src"}, "", nil),
 
-			expectedCommits: []*models.Commit{},
-			expectedError:   nil,
+			expectedCommitOpts: []models.NewCommitOpts{},
+			expectedError:      nil,
 		},
 	}
 
 	for _, scenario := range scenarios {
 		t.Run(scenario.testName, func(t *testing.T) {
-			common := utils.NewDummyCommon()
+			common := common.NewDummyCommon()
 			common.AppState = &config.AppState{}
 			common.AppState.GitLogOrder = scenario.logOrder
 			cmd := oscommands.NewDummyCmdObjBuilder(scenario.runner)
 
 			builder := &CommitLoader{
-				Common:        common,
-				cmd:           cmd,
-				getRebaseMode: func() (enums.RebaseMode, error) { return scenario.rebaseMode, nil },
-				dotGitDir:     ".git",
+				Common:              common,
+				cmd:                 cmd,
+				getWorkingTreeState: func() models.WorkingTreeState { return models.WorkingTreeState{} },
+				dotGitDir:           ".git",
 				readFile: func(filename string) ([]byte, error) {
 					return []byte(""), nil
 				},
@@ -322,12 +315,17 @@ func TestGetCommits(t *testing.T) {
 				},
 			}
 
+			hashPool := &utils.StringPool{}
+
 			common.UserConfig().Git.MainBranches = scenario.mainBranches
 			opts := scenario.opts
 			opts.MainBranches = NewMainBranches(common, cmd)
+			opts.HashPool = hashPool
 			commits, err := builder.GetCommits(opts)
 
-			assert.Equal(t, scenario.expectedCommits, commits)
+			expectedCommits := lo.Map(scenario.expectedCommitOpts,
+				func(opts models.NewCommitOpts, _ int) *models.Commit { return models.NewCommit(hashPool, opts) })
+			assert.Equal(t, expectedCommits, commits)
 			assert.Equal(t, scenario.expectedError, err)
 
 			scenario.runner.CheckForMissingCalls()
@@ -336,19 +334,22 @@ func TestGetCommits(t *testing.T) {
 }
 
 func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
+	hashPool := &utils.StringPool{}
+
 	scenarios := []struct {
-		testName        string
-		todos           []todo.Todo
-		doneTodos       []todo.Todo
-		amendFileExists bool
-		expectedHash    string
+		testName          string
+		todos             []todo.Todo
+		doneTodos         []todo.Todo
+		amendFileExists   bool
+		messageFileExists bool
+		expectedResult    *models.Commit
 	}{
 		{
 			testName:        "no done todos",
 			todos:           []todo.Todo{},
 			doneTodos:       []todo.Todo{},
 			amendFileExists: false,
-			expectedHash:    "",
+			expectedResult:  nil,
 		},
 		{
 			testName: "common case (conflict)",
@@ -364,7 +365,11 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 				},
 			},
 			amendFileExists: false,
-			expectedHash:    "fa1afe1",
+			expectedResult: models.NewCommit(hashPool, models.NewCommitOpts{
+				Hash:   "fa1afe1",
+				Action: todo.Pick,
+				Status: models.StatusConflicted,
+			}),
 		},
 		{
 			testName: "last command was 'break'",
@@ -373,7 +378,7 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 				{Command: todo.Break},
 			},
 			amendFileExists: false,
-			expectedHash:    "",
+			expectedResult:  nil,
 		},
 		{
 			testName: "last command was 'exec'",
@@ -385,7 +390,7 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 				},
 			},
 			amendFileExists: false,
-			expectedHash:    "",
+			expectedResult:  nil,
 		},
 		{
 			testName: "last command was 'reword'",
@@ -394,7 +399,7 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 				{Command: todo.Reword},
 			},
 			amendFileExists: false,
-			expectedHash:    "",
+			expectedResult:  nil,
 		},
 		{
 			testName: "'pick' was rescheduled",
@@ -411,7 +416,7 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 				},
 			},
 			amendFileExists: false,
-			expectedHash:    "",
+			expectedResult:  nil,
 		},
 		{
 			testName: "'pick' was rescheduled, buggy git version",
@@ -436,7 +441,7 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 				},
 			},
 			amendFileExists: false,
-			expectedHash:    "",
+			expectedResult:  nil,
 		},
 		{
 			testName: "conflicting 'pick' after 'exec'",
@@ -461,7 +466,11 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 				},
 			},
 			amendFileExists: false,
-			expectedHash:    "fa1afe1",
+			expectedResult: models.NewCommit(hashPool, models.NewCommitOpts{
+				Hash:   "fa1afe1",
+				Action: todo.Pick,
+				Status: models.StatusConflicted,
+			}),
 		},
 		{
 			testName: "'edit' with amend file",
@@ -473,10 +482,10 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 				},
 			},
 			amendFileExists: true,
-			expectedHash:    "",
+			expectedResult:  nil,
 		},
 		{
-			testName: "'edit' without amend file",
+			testName: "'edit' without amend file but message file",
 			todos:    []todo.Todo{},
 			doneTodos: []todo.Todo{
 				{
@@ -484,19 +493,37 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 					Commit:  "fa1afe1",
 				},
 			},
-			amendFileExists: false,
-			expectedHash:    "fa1afe1",
+			amendFileExists:   false,
+			messageFileExists: true,
+			expectedResult: models.NewCommit(hashPool, models.NewCommitOpts{
+				Hash:   "fa1afe1",
+				Action: todo.Edit,
+				Status: models.StatusConflicted,
+			}),
+		},
+		{
+			testName: "'edit' without amend and without message file",
+			todos:    []todo.Todo{},
+			doneTodos: []todo.Todo{
+				{
+					Command: todo.Edit,
+					Commit:  "fa1afe1",
+				},
+			},
+			amendFileExists:   false,
+			messageFileExists: false,
+			expectedResult:    nil,
 		},
 	}
 	for _, scenario := range scenarios {
 		t.Run(scenario.testName, func(t *testing.T) {
-			common := utils.NewDummyCommon()
+			common := common.NewDummyCommon()
 
 			builder := &CommitLoader{
-				Common:        common,
-				cmd:           oscommands.NewDummyCmdObjBuilder(oscommands.NewFakeRunner(t)),
-				getRebaseMode: func() (enums.RebaseMode, error) { return enums.REBASE_MODE_INTERACTIVE, nil },
-				dotGitDir:     ".git",
+				Common:              common,
+				cmd:                 oscommands.NewDummyCmdObjBuilder(oscommands.NewFakeRunner(t)),
+				getWorkingTreeState: func() models.WorkingTreeState { return models.WorkingTreeState{Rebasing: true} },
+				dotGitDir:           ".git",
 				readFile: func(filename string) ([]byte, error) {
 					return []byte(""), nil
 				},
@@ -505,30 +532,30 @@ func TestCommitLoader_getConflictedCommitImpl(t *testing.T) {
 				},
 			}
 
-			hash := builder.getConflictedCommitImpl(scenario.todos, scenario.doneTodos, scenario.amendFileExists)
-			assert.Equal(t, scenario.expectedHash, hash)
+			hash := builder.getConflictedCommitImpl(hashPool, scenario.todos, scenario.doneTodos, scenario.amendFileExists, scenario.messageFileExists)
+			assert.Equal(t, scenario.expectedResult, hash)
 		})
 	}
 }
 
 func TestCommitLoader_setCommitMergedStatuses(t *testing.T) {
 	type scenario struct {
-		testName        string
-		commits         []*models.Commit
-		ancestor        string
-		expectedCommits []*models.Commit
+		testName           string
+		commitOpts         []models.NewCommitOpts
+		ancestor           string
+		expectedCommitOpts []models.NewCommitOpts
 	}
 
 	scenarios := []scenario{
 		{
 			testName: "basic",
-			commits: []*models.Commit{
+			commitOpts: []models.NewCommitOpts{
 				{Hash: "12345", Name: "1", Action: models.ActionNone, Status: models.StatusUnpushed},
 				{Hash: "67890", Name: "2", Action: models.ActionNone, Status: models.StatusPushed},
 				{Hash: "abcde", Name: "3", Action: models.ActionNone, Status: models.StatusPushed},
 			},
 			ancestor: "67890",
-			expectedCommits: []*models.Commit{
+			expectedCommitOpts: []models.NewCommitOpts{
 				{Hash: "12345", Name: "1", Action: models.ActionNone, Status: models.StatusUnpushed},
 				{Hash: "67890", Name: "2", Action: models.ActionNone, Status: models.StatusMerged},
 				{Hash: "abcde", Name: "3", Action: models.ActionNone, Status: models.StatusMerged},
@@ -536,13 +563,13 @@ func TestCommitLoader_setCommitMergedStatuses(t *testing.T) {
 		},
 		{
 			testName: "with update-ref",
-			commits: []*models.Commit{
+			commitOpts: []models.NewCommitOpts{
 				{Hash: "12345", Name: "1", Action: models.ActionNone, Status: models.StatusUnpushed},
 				{Hash: "", Name: "", Action: todo.UpdateRef, Status: models.StatusNone},
 				{Hash: "abcde", Name: "3", Action: models.ActionNone, Status: models.StatusPushed},
 			},
 			ancestor: "deadbeef",
-			expectedCommits: []*models.Commit{
+			expectedCommitOpts: []models.NewCommitOpts{
 				{Hash: "12345", Name: "1", Action: models.ActionNone, Status: models.StatusUnpushed},
 				{Hash: "", Name: "", Action: todo.UpdateRef, Status: models.StatusNone},
 				{Hash: "abcde", Name: "3", Action: models.ActionNone, Status: models.StatusPushed},
@@ -552,9 +579,14 @@ func TestCommitLoader_setCommitMergedStatuses(t *testing.T) {
 
 	for _, scenario := range scenarios {
 		t.Run(scenario.testName, func(t *testing.T) {
-			expectedCommits := scenario.commits
-			setCommitMergedStatuses(scenario.ancestor, expectedCommits)
-			assert.Equal(t, scenario.expectedCommits, expectedCommits)
+			hashPool := &utils.StringPool{}
+
+			commits := lo.Map(scenario.commitOpts,
+				func(opts models.NewCommitOpts, _ int) *models.Commit { return models.NewCommit(hashPool, opts) })
+			setCommitMergedStatuses(scenario.ancestor, commits)
+			expectedCommits := lo.Map(scenario.expectedCommitOpts,
+				func(opts models.NewCommitOpts, _ int) *models.Commit { return models.NewCommit(hashPool, opts) })
+			assert.Equal(t, expectedCommits, commits)
 		})
 	}
 }
