@@ -7,10 +7,8 @@ import (
 	gogit "github.com/jesseduffield/go-git/v5"
 	"github.com/jesseduffield/lazygit/pkg/commands/git_config"
 	"github.com/jesseduffield/lazygit/pkg/commands/oscommands"
-	"github.com/jesseduffield/lazygit/pkg/commands/patch"
 	"github.com/jesseduffield/lazygit/pkg/common"
 	"github.com/jesseduffield/lazygit/pkg/config"
-	"github.com/jesseduffield/lazygit/pkg/utils"
 	"github.com/spf13/afero"
 )
 
@@ -33,7 +31,7 @@ func buildGitCommon(deps commonDeps) *GitCommon {
 
 	gitCommon.Common = deps.common
 	if gitCommon.Common == nil {
-		gitCommon.Common = utils.NewDummyCommonWithUserConfigAndAppState(deps.userConfig, deps.appState)
+		gitCommon.Common = common.NewDummyCommonWithUserConfigAndAppState(deps.userConfig, deps.appState)
 	}
 
 	if deps.fs != nil {
@@ -126,26 +124,6 @@ func buildWorkingTreeCommands(deps commonDeps) *WorkingTreeCommands {
 	return NewWorkingTreeCommands(gitCommon, submoduleCommands, fileLoader)
 }
 
-func buildPatchCommands(deps commonDeps) *PatchCommands { //nolint:golint,unused
-	gitCommon := buildGitCommon(deps)
-	rebaseCommands := buildRebaseCommands(deps)
-	commitCommands := buildCommitCommands(deps)
-	statusCommands := buildStatusCommands(deps)
-	stashCommands := buildStashCommands(deps)
-	loadFileFn := func(from string, to string, reverse bool, filename string, plain bool) (string, error) {
-		return "", nil
-	}
-	patchBuilder := patch.NewPatchBuilder(gitCommon.Log, loadFileFn)
-
-	return NewPatchCommands(gitCommon, rebaseCommands, commitCommands, statusCommands, stashCommands, patchBuilder)
-}
-
-func buildStatusCommands(deps commonDeps) *StatusCommands { //nolint:golint,unused
-	gitCommon := buildGitCommon(deps)
-
-	return NewStatusCommands(gitCommon)
-}
-
 func buildStashCommands(deps commonDeps) *StashCommands {
 	gitCommon := buildGitCommon(deps)
 	fileLoader := buildFileLoader(gitCommon)
