@@ -53,9 +53,9 @@ func (self *RebaseCommands) RewordCommit(commits []*models.Commit, index int, su
 	return self.ContinueRebase()
 }
 
-func (self *RebaseCommands) RewordCommitInEditor(commits []*models.Commit, index int) (oscommands.ICmdObj, error) {
+func (self *RebaseCommands) RewordCommitInEditor(commits []*models.Commit, index int) (*oscommands.CmdObj, error) {
 	changes := []daemon.ChangeTodoAction{{
-		Hash:      commits[index].Hash,
+		Hash:      commits[index].Hash(),
 		NewAction: todo.Reword,
 	}}
 	self.os.LogCommand(logTodoChanges(changes), false)
@@ -80,7 +80,7 @@ func (self *RebaseCommands) SetCommitAuthor(commits []*models.Commit, start, end
 
 func (self *RebaseCommands) AddCommitCoAuthor(commits []*models.Commit, start, end int, value string) error {
 	return self.GenericAmend(commits, start, end, func(commit *models.Commit) error {
-		return self.commit.AddCoAuthor(commit.Hash, value)
+		return self.commit.AddCoAuthor(commit.Hash(), value)
 	})
 }
 
@@ -113,7 +113,7 @@ func (self *RebaseCommands) MoveCommitsDown(commits []*models.Commit, startIdx i
 	baseHashOrRoot := getBaseHashOrRoot(commits, endIdx+2)
 
 	hashes := lo.Map(commits[startIdx:endIdx+1], func(commit *models.Commit, _ int) string {
-		return commit.Hash
+		return commit.Hash()
 	})
 
 	return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
@@ -127,7 +127,7 @@ func (self *RebaseCommands) MoveCommitsUp(commits []*models.Commit, startIdx int
 	baseHashOrRoot := getBaseHashOrRoot(commits, endIdx+1)
 
 	hashes := lo.Map(commits[startIdx:endIdx+1], func(commit *models.Commit, _ int) string {
-		return commit.Hash
+		return commit.Hash()
 	})
 
 	return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
@@ -147,7 +147,7 @@ func (self *RebaseCommands) InteractiveRebase(commits []*models.Commit, startIdx
 
 	changes := lo.FilterMap(commits[startIdx:endIdx+1], func(commit *models.Commit, _ int) (daemon.ChangeTodoAction, bool) {
 		return daemon.ChangeTodoAction{
-			Hash:      commit.Hash,
+			Hash:      commit.Hash(),
 			NewAction: action,
 		}, !commit.IsMerge()
 	})
@@ -209,7 +209,7 @@ type PrepareInteractiveRebaseCommandOpts struct {
 // PrepareInteractiveRebaseCommand returns the cmd for an interactive rebase
 // we tell git to run lazygit to edit the todo list, and we pass the client
 // lazygit instructions what to do with the todo file
-func (self *RebaseCommands) PrepareInteractiveRebaseCommand(opts PrepareInteractiveRebaseCommandOpts) oscommands.ICmdObj {
+func (self *RebaseCommands) PrepareInteractiveRebaseCommand(opts PrepareInteractiveRebaseCommandOpts) *oscommands.CmdObj {
 	ex := oscommands.GetLazygitPath()
 
 	cmdArgs := NewGitCmd("rebase").
@@ -293,7 +293,7 @@ func (self *RebaseCommands) getHashOfLastCommitMade() (string, error) {
 func (self *RebaseCommands) AmendTo(commits []*models.Commit, commitIndex int) error {
 	commit := commits[commitIndex]
 
-	if err := self.commit.CreateFixupCommit(commit.Hash); err != nil {
+	if err := self.commit.CreateFixupCommit(commit.Hash()); err != nil {
 		return err
 	}
 
@@ -305,7 +305,7 @@ func (self *RebaseCommands) AmendTo(commits []*models.Commit, commitIndex int) e
 	return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
 		baseHashOrRoot: getBaseHashOrRoot(commits, commitIndex+1),
 		overrideEditor: true,
-		instruction:    daemon.NewMoveFixupCommitDownInstruction(commit.Hash, fixupHash, true),
+		instruction:    daemon.NewMoveFixupCommitDownInstruction(commit.Hash(), fixupHash, true),
 	}).Run()
 }
 
@@ -318,7 +318,7 @@ func (self *RebaseCommands) MoveFixupCommitDown(commits []*models.Commit, target
 	return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
 		baseHashOrRoot: getBaseHashOrRoot(commits, targetCommitIndex+1),
 		overrideEditor: true,
-		instruction:    daemon.NewMoveFixupCommitDownInstruction(commits[targetCommitIndex].Hash, fixupHash, false),
+		instruction:    daemon.NewMoveFixupCommitDownInstruction(commits[targetCommitIndex].Hash(), fixupHash, false),
 	}).Run()
 }
 
@@ -326,7 +326,7 @@ func todoFromCommit(commit *models.Commit) utils.Todo {
 	if commit.Action == todo.UpdateRef {
 		return utils.Todo{Ref: commit.Name}
 	} else {
-		return utils.Todo{Hash: commit.Hash}
+		return utils.Todo{Hash: commit.Hash()}
 	}
 }
 
@@ -334,7 +334,7 @@ func todoFromCommit(commit *models.Commit) utils.Todo {
 func (self *RebaseCommands) EditRebaseTodo(commits []*models.Commit, action todo.TodoCommand) error {
 	commitsWithAction := lo.Map(commits, func(commit *models.Commit, _ int) utils.TodoChange {
 		return utils.TodoChange{
-			Hash:      commit.Hash,
+			Hash:      commit.Hash(),
 			NewAction: action,
 		}
 	})
@@ -383,7 +383,7 @@ func (self *RebaseCommands) MoveTodosUp(commits []*models.Commit) error {
 
 // SquashAllAboveFixupCommits squashes all fixup! commits above the given one
 func (self *RebaseCommands) SquashAllAboveFixupCommits(commit *models.Commit) error {
-	hashOrRoot := commit.Hash + "^"
+	hashOrRoot := commit.Hash() + "^"
 	if commit.IsFirstCommit() {
 		hashOrRoot = "--root"
 	}
@@ -400,7 +400,19 @@ func (self *RebaseCommands) SquashAllAboveFixupCommits(commit *models.Commit) er
 func (self *RebaseCommands) BeginInteractiveRebaseForCommit(
 	commits []*models.Commit, commitIndex int, keepCommitsThatBecomeEmpty bool,
 ) error {
-	return self.BeginInteractiveRebaseForCommitRange(commits, commitIndex, commitIndex, keepCommitsThatBecomeEmpty)
+	if commitIndex < len(commits) && commits[commitIndex].IsMerge() {
+		if self.config.NeedsGpgSubprocessForCommit() {
+			return errors.New(self.Tr.DisabledForGPG)
+		}
+
+		return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
+			baseHashOrRoot:             getBaseHashOrRoot(commits, commitIndex),
+			instruction:                daemon.NewInsertBreakInstruction(),
+			keepCommitsThatBecomeEmpty: keepCommitsThatBecomeEmpty,
+		}).Run()
+	} else {
+		return self.BeginInteractiveRebaseForCommitRange(commits, commitIndex, commitIndex, keepCommitsThatBecomeEmpty)
+	}
 }
 
 func (self *RebaseCommands) BeginInteractiveRebaseForCommitRange(
@@ -420,7 +432,7 @@ func (self *RebaseCommands) BeginInteractiveRebaseForCommitRange(
 	changes := make([]daemon.ChangeTodoAction, 0, end-start)
 	for commitIndex := end; commitIndex >= start; commitIndex-- {
 		changes = append(changes, daemon.ChangeTodoAction{
-			Hash:      commits[commitIndex].Hash,
+			Hash:      commits[commitIndex].Hash(),
 			NewAction: todo.Edit,
 		})
 	}
@@ -446,7 +458,7 @@ func (self *RebaseCommands) RebaseBranchFromBaseCommit(targetBranchName string,
 	}).Run()
 }
 
-func (self *RebaseCommands) GenericMergeOrRebaseActionCmdObj(commandType string, command string) oscommands.ICmdObj {
+func (self *RebaseCommands) GenericMergeOrRebaseActionCmdObj(commandType string, command string) *oscommands.CmdObj {
 	cmdArgs := NewGitCmd(commandType).Arg("--" + command).ToArgv()
 
 	return self.cmd.New(cmdArgs)
@@ -485,7 +497,7 @@ func (self *RebaseCommands) GenericMergeOrRebaseAction(commandType string, comma
 	return nil
 }
 
-func (self *RebaseCommands) runSkipEditorCommand(cmdObj oscommands.ICmdObj) error {
+func (self *RebaseCommands) runSkipEditorCommand(cmdObj *oscommands.CmdObj) error {
 	instruction := daemon.NewExitImmediatelyInstruction()
 	lazyGitPath := oscommands.GetLazygitPath()
 	return cmdObj.
@@ -533,41 +545,21 @@ func (self *RebaseCommands) DiscardOldFileChanges(commits []*models.Commit, comm
 
 // CherryPickCommits begins an interactive rebase with the given hashes being cherry picked onto HEAD
 func (self *RebaseCommands) CherryPickCommits(commits []*models.Commit) error {
-	commitLines := lo.Map(commits, func(commit *models.Commit, _ int) string {
-		return fmt.Sprintf("%s %s", utils.ShortHash(commit.Hash), commit.Name)
-	})
-	msg := utils.ResolvePlaceholderString(
-		self.Tr.Log.CherryPickCommits,
-		map[string]string{
-			"commitLines": strings.Join(commitLines, "\n"),
-		},
-	)
-	self.os.LogCommand(msg, false)
+	hasMergeCommit := lo.SomeBy(commits, func(c *models.Commit) bool { return c.IsMerge() })
+	cmdArgs := NewGitCmd("cherry-pick").
+		Arg("--allow-empty").
+		ArgIf(self.version.IsAtLeast(2, 45, 0), "--empty=keep", "--keep-redundant-commits").
+		ArgIf(hasMergeCommit, "-m1").
+		Arg(lo.Reverse(lo.Map(commits, func(c *models.Commit, _ int) string { return c.Hash() }))...).
+		ToArgv()
 
-	return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
-		baseHashOrRoot: "HEAD",
-		instruction:    daemon.NewCherryPickCommitsInstruction(commits),
-	}).Run()
-}
-
-// CherryPickCommitsDuringRebase simply prepends the given commits to the existing git-rebase-todo file
-func (self *RebaseCommands) CherryPickCommitsDuringRebase(commits []*models.Commit) error {
-	todoLines := lo.Map(commits, func(commit *models.Commit, _ int) daemon.TodoLine {
-		return daemon.TodoLine{
-			Action: "pick",
-			Commit: commit,
-		}
-	})
-
-	todo := daemon.TodoLinesToString(todoLines)
-	filePath := filepath.Join(self.repoPaths.worktreeGitDirPath, "rebase-merge/git-rebase-todo")
-	return utils.PrependStrToTodoFile(filePath, []byte(todo))
+	return self.cmd.New(cmdArgs).Run()
 }
 
 func (self *RebaseCommands) DropMergeCommit(commits []*models.Commit, commitIndex int) error {
 	return self.PrepareInteractiveRebaseCommand(PrepareInteractiveRebaseCommandOpts{
 		baseHashOrRoot: getBaseHashOrRoot(commits, commitIndex+1),
-		instruction:    daemon.NewDropMergeCommitInstruction(commits[commitIndex].Hash),
+		instruction:    daemon.NewDropMergeCommitInstruction(commits[commitIndex].Hash()),
 	}).Run()
 }
 
@@ -579,7 +571,7 @@ func getBaseHashOrRoot(commits []*models.Commit, index int) string {
 	// be starting a rebase from 300 commits ago (which is the original commit limit
 	// at time of writing)
 	if index < len(commits) {
-		return commits[index].Hash
+		return commits[index].Hash()
 	} else {
 		return "--root"
 	}
