@@ -4,6 +4,7 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"regexp"
 
 	"github.com/go-errors/errors"
 	"github.com/jesseduffield/lazygit/pkg/commands/models"
@@ -28,17 +29,21 @@ func NewWorkingTreeCommands(
 	}
 }
 
-func (self *WorkingTreeCommands) OpenMergeToolCmdObj() oscommands.ICmdObj {
+func (self *WorkingTreeCommands) OpenMergeToolCmdObj() *oscommands.CmdObj {
 	return self.cmd.New(NewGitCmd("mergetool").ToArgv())
 }
 
 // StageFile stages a file
 func (self *WorkingTreeCommands) StageFile(path string) error {
-	return self.StageFiles([]string{path})
+	return self.StageFiles([]string{path}, nil)
 }
 
-func (self *WorkingTreeCommands) StageFiles(paths []string) error {
-	cmdArgs := NewGitCmd("add").Arg("--").Arg(paths...).ToArgv()
+func (self *WorkingTreeCommands) StageFiles(paths []string, extraArgs []string) error {
+	cmdArgs := NewGitCmd("add").
+		Arg(extraArgs...).
+		Arg("--").
+		Arg(paths...).
+		ToArgv()
 
 	return self.cmd.New(cmdArgs).Run()
 }
@@ -88,11 +93,11 @@ func (self *WorkingTreeCommands) BeforeAndAfterFileForRename(file *models.File)
 	var beforeFile *models.File
 	var afterFile *models.File
 	for _, f := range filesWithoutRenames {
-		if f.Name == file.PreviousName {
+		if f.Path == file.PreviousPath {
 			beforeFile = f
 		}
 
-		if f.Name == file.Name {
+		if f.Path == file.Path {
 			afterFile = f
 		}
 	}
@@ -109,10 +114,6 @@ func (self *WorkingTreeCommands) BeforeAndAfterFileForRename(file *models.File)
 	return beforeFile, afterFile, nil
 }
 
-func newCheckoutCommand() *GitCommandBuilder {
-	return NewGitCmd("checkout").Config(fmt.Sprintf("core.hooksPath=%s", os.DevNull))
-}
-
 // DiscardAllFileChanges directly
 func (self *WorkingTreeCommands) DiscardAllFileChanges(file *models.File) error {
 	if file.IsRename() {
@@ -134,13 +135,13 @@ func (self *WorkingTreeCommands) DiscardAllFileChanges(file *models.File) error
 
 	if file.ShortStatus == "AA" {
 		if err := self.cmd.New(
-			newCheckoutCommand().Arg("--ours", "--", file.Name).ToArgv(),
+			NewGitCmd("checkout").Arg("--ours", "--", file.Path).ToArgv(),
 		).Run(); err != nil {
 			return err
 		}
 
 		if err := self.cmd.New(
-			NewGitCmd("add").Arg("--", file.Name).ToArgv(),
+			NewGitCmd("add").Arg("--", file.Path).ToArgv(),
 		).Run(); err != nil {
 			return err
 		}
@@ -149,14 +150,14 @@ func (self *WorkingTreeCommands) DiscardAllFileChanges(file *models.File) error
 
 	if file.ShortStatus == "DU" {
 		return self.cmd.New(
-			NewGitCmd("rm").Arg("--", file.Name).ToArgv(),
+			NewGitCmd("rm").Arg("--", file.Path).ToArgv(),
 		).Run()
 	}
 
 	// if the file isn't tracked, we assume you want to delete it
 	if file.HasStagedChanges || file.HasMergeConflicts {
 		if err := self.cmd.New(
-			NewGitCmd("reset").Arg("--", file.Name).ToArgv(),
+			NewGitCmd("reset").Arg("--", file.Path).ToArgv(),
 		).Run(); err != nil {
 			return err
 		}
@@ -167,7 +168,7 @@ func (self *WorkingTreeCommands) DiscardAllFileChanges(file *models.File) error
 	}
 
 	if file.Added {
-		return self.os.RemoveFile(file.Name)
+		return self.os.RemoveFile(file.Path)
 	}
 
 	return self.DiscardUnstagedFileChanges(file)
@@ -193,13 +194,13 @@ func (self *WorkingTreeCommands) DiscardUnstagedDirChanges(node IFileNode) error
 			return err
 		}
 
-		cmdArgs := newCheckoutCommand().Arg("--", node.GetPath()).ToArgv()
+		cmdArgs := NewGitCmd("checkout").Arg("--", node.GetPath()).ToArgv()
 		if err := self.cmd.New(cmdArgs).Run(); err != nil {
 			return err
 		}
 	} else {
 		if file.Added && !file.HasStagedChanges {
-			return self.os.RemoveFile(file.Name)
+			return self.os.RemoveFile(file.Path)
 		}
 
 		if err := self.DiscardUnstagedFileChanges(file); err != nil {
@@ -226,19 +227,25 @@ func (self *WorkingTreeCommands) RemoveUntrackedDirFiles(node IFileNode) error {
 }
 
 func (self *WorkingTreeCommands) DiscardUnstagedFileChanges(file *models.File) error {
-	cmdArgs := newCheckoutCommand().Arg("--", file.Name).ToArgv()
+	cmdArgs := NewGitCmd("checkout").Arg("--", file.Path).ToArgv()
 	return self.cmd.New(cmdArgs).Run()
 }
 
+// Escapes special characters in a filename for gitignore and exclude files
+func escapeFilename(filename string) string {
+	re := regexp.MustCompile(`^[!#]|[\[\]*]`)
+	return re.ReplaceAllString(filename, `\${0}`)
+}
+
 // Ignore adds a file to the gitignore for the repo
 func (self *WorkingTreeCommands) Ignore(filename string) error {
-	return self.os.AppendLineToFile(".gitignore", filename)
+	return self.os.AppendLineToFile(".gitignore", escapeFilename(filename))
 }
 
 // Exclude adds a file to the .git/info/exclude for the repo
 func (self *WorkingTreeCommands) Exclude(filename string) error {
 	excludeFile := filepath.Join(self.repoPaths.repoGitDirPath, "info", "exclude")
-	return self.os.AppendLineToFile(excludeFile, filename)
+	return self.os.AppendLineToFile(excludeFile, escapeFilename(filename))
 }
 
 // WorktreeFileDiff returns the diff of a file
@@ -248,7 +255,7 @@ func (self *WorkingTreeCommands) WorktreeFileDiff(file *models.File, plain bool,
 	return s
 }
 
-func (self *WorkingTreeCommands) WorktreeFileDiffCmdObj(node models.IFile, plain bool, cached bool) oscommands.ICmdObj {
+func (self *WorkingTreeCommands) WorktreeFileDiffCmdObj(node models.IFile, plain bool, cached bool) *oscommands.CmdObj {
 	colorArg := self.UserConfig().Git.Paging.ColorArg
 	if plain {
 		colorArg = "never"
@@ -286,7 +293,7 @@ func (self *WorkingTreeCommands) ShowFileDiff(from string, to string, reverse bo
 	return self.ShowFileDiffCmdObj(from, to, reverse, fileName, plain).RunWithOutput()
 }
 
-func (self *WorkingTreeCommands) ShowFileDiffCmdObj(from string, to string, reverse bool, fileName string, plain bool) oscommands.ICmdObj {
+func (self *WorkingTreeCommands) ShowFileDiffCmdObj(from string, to string, reverse bool, fileName string, plain bool) *oscommands.CmdObj {
 	contextSize := self.AppState.DiffContextSize
 
 	colorArg := self.UserConfig().Git.Paging.ColorArg
@@ -319,7 +326,7 @@ func (self *WorkingTreeCommands) ShowFileDiffCmdObj(from string, to string, reve
 
 // CheckoutFile checks out the file for the given commit
 func (self *WorkingTreeCommands) CheckoutFile(commitHash, fileName string) error {
-	cmdArgs := newCheckoutCommand().Arg(commitHash, "--", fileName).
+	cmdArgs := NewGitCmd("checkout").Arg(commitHash, "--", fileName).
 		ToArgv()
 
 	return self.cmd.New(cmdArgs).Run()
@@ -327,7 +334,7 @@ func (self *WorkingTreeCommands) CheckoutFile(commitHash, fileName string) error
 
 // DiscardAnyUnstagedFileChanges discards any unstaged file changes via `git checkout -- .`
 func (self *WorkingTreeCommands) DiscardAnyUnstagedFileChanges() error {
-	cmdArgs := newCheckoutCommand().Arg("--", ".").
+	cmdArgs := NewGitCmd("checkout").Arg("--", ".").
 		ToArgv()
 
 	return self.cmd.New(cmdArgs).Run()
@@ -341,6 +348,13 @@ func (self *WorkingTreeCommands) RemoveTrackedFiles(name string) error {
 	return self.cmd.New(cmdArgs).Run()
 }
 
+func (self *WorkingTreeCommands) RemoveConflictedFile(name string) error {
+	cmdArgs := NewGitCmd("rm").Arg("--", name).
+		ToArgv()
+
+	return self.cmd.New(cmdArgs).Run()
+}
+
 // RemoveUntrackedFiles runs `git clean -fd`
 func (self *WorkingTreeCommands) RemoveUntrackedFiles() error {
 	cmdArgs := NewGitCmd("clean").Arg("-fd").ToArgv()
