@@ -22,6 +22,8 @@ var CONTEXT_KEYS_SHOWING_DIFFS = []types.ContextKey{
 	context.STAGING_SECONDARY_CONTEXT_KEY,
 	context.PATCH_BUILDING_MAIN_CONTEXT_KEY,
 	context.PATCH_BUILDING_SECONDARY_CONTEXT_KEY,
+	context.NORMAL_MAIN_CONTEXT_KEY,
+	context.NORMAL_SECONDARY_CONTEXT_KEY,
 }
 
 type ContextLinesController struct {
@@ -97,7 +99,7 @@ func (self *ContextLinesController) applyChange() error {
 	self.c.Toast(fmt.Sprintf(self.c.Tr.DiffContextSizeChanged, self.c.AppState.DiffContextSize))
 	self.c.SaveAppStateAndLogError()
 
-	currentContext := self.c.Context().CurrentStatic()
+	currentContext := self.currentSidePanel()
 	switch currentContext.GetKey() {
 	// we make an exception for our staging and patch building contexts because they actually need to refresh their state afterwards.
 	case context.PATCH_BUILDING_MAIN_CONTEXT_KEY:
@@ -121,6 +123,18 @@ func (self *ContextLinesController) checkCanChangeContext() error {
 func (self *ContextLinesController) isShowingDiff() bool {
 	return lo.Contains(
 		CONTEXT_KEYS_SHOWING_DIFFS,
-		self.c.Context().CurrentStatic().GetKey(),
+		self.currentSidePanel().GetKey(),
 	)
 }
+
+func (self *ContextLinesController) currentSidePanel() types.Context {
+	currentContext := self.c.Context().CurrentStatic()
+	if currentContext.GetKey() == context.NORMAL_MAIN_CONTEXT_KEY ||
+		currentContext.GetKey() == context.NORMAL_SECONDARY_CONTEXT_KEY {
+		if sidePanelContext := self.c.Context().NextInStack(currentContext); sidePanelContext != nil {
+			return sidePanelContext
+		}
+	}
+
+	return currentContext
+}
