@@ -10,6 +10,7 @@ import (
 	"github.com/jesseduffield/lazygit/pkg/config"
 	"github.com/jesseduffield/lazygit/pkg/gui/context"
 	"github.com/jesseduffield/lazygit/pkg/gui/types"
+	"github.com/samber/lo"
 )
 
 type WorkingTreeHelper struct {
@@ -34,25 +35,27 @@ func NewWorkingTreeHelper(
 }
 
 func (self *WorkingTreeHelper) AnyStagedFiles() bool {
-	for _, file := range self.c.Model().Files {
-		if file.HasStagedChanges {
-			return true
-		}
-	}
-	return false
+	return AnyStagedFiles(self.c.Model().Files)
+}
+
+func AnyStagedFiles(files []*models.File) bool {
+	return lo.SomeBy(files, func(f *models.File) bool { return f.HasStagedChanges })
 }
 
 func (self *WorkingTreeHelper) AnyTrackedFiles() bool {
-	for _, file := range self.c.Model().Files {
-		if file.Tracked {
-			return true
-		}
-	}
-	return false
+	return AnyTrackedFiles(self.c.Model().Files)
+}
+
+func AnyTrackedFiles(files []*models.File) bool {
+	return lo.SomeBy(files, func(f *models.File) bool { return f.Tracked })
 }
 
 func (self *WorkingTreeHelper) IsWorkingTreeDirty() bool {
-	return self.AnyStagedFiles() || self.AnyTrackedFiles()
+	return IsWorkingTreeDirty(self.c.Model().Files)
+}
+
+func IsWorkingTreeDirty(files []*models.File) bool {
+	return AnyStagedFiles(files) || AnyTrackedFiles(files)
 }
 
 func (self *WorkingTreeHelper) FileForSubmodule(submodule *models.SubmoduleConfig) *models.File {
@@ -109,7 +112,7 @@ func (self *WorkingTreeHelper) handleCommit(summary string, description string,
 	self.c.LogAction(self.c.Tr.Actions.Commit)
 	return self.gpgHelper.WithGpgHandling(cmdObj, git_commands.CommitGpgSign, self.c.Tr.CommittingStatus,
 		func() error {
-			self.commitsHelper.OnCommitSuccess()
+			self.commitsHelper.ClearPreservedCommitMessage()
 			return nil
 		}, nil)
 }
@@ -121,7 +124,7 @@ func (self *WorkingTreeHelper) switchFromCommitMessagePanelToEditor(filepath str
 	// access to the last message that the user typed, and it might be very
 	// different from what was last in the commit panel. So the best we can do
 	// here is to always clear the remembered commit message.
-	self.commitsHelper.OnCommitSuccess()
+	self.commitsHelper.ClearPreservedCommitMessage()
 
 	self.c.LogAction(self.c.Tr.Actions.Commit)
 	return self.c.RunSubprocessAndRefresh(
@@ -133,6 +136,10 @@ func (self *WorkingTreeHelper) switchFromCommitMessagePanelToEditor(filepath str
 // their editor rather than via the popup panel
 func (self *WorkingTreeHelper) HandleCommitEditorPress() error {
 	return self.WithEnsureCommittableFiles(func() error {
+		// See reasoning in switchFromCommitMessagePanelToEditor for why it makes sense
+		// to clear this message before calling into the editor
+		self.commitsHelper.ClearPreservedCommitMessage()
+
 		self.c.LogAction(self.c.Tr.Actions.Commit)
 		return self.c.RunSubprocessAndRefresh(
 			self.c.Git().Commit.CommitEditorCmdObj(),
