@@ -20,7 +20,7 @@ func (gui *Gui) scrollDownView(view *gocui.View) {
 	scrollHeight := gui.c.UserConfig().Gui.ScrollHeight
 	view.ScrollDown(scrollHeight)
 
-	if manager, ok := gui.viewBufferManagerMap[view.Name()]; ok {
+	if manager := gui.getViewBufferManagerForView(view); manager != nil {
 		manager.ReadLines(scrollHeight)
 	}
 }
@@ -131,7 +131,7 @@ func (gui *Gui) pageDownConfirmationPanel() error {
 
 func (gui *Gui) goToConfirmationPanelTop() error {
 	if gui.Views.Confirmation.Editable {
-		return nil
+		return gocui.ErrKeybindingNotHandled
 	}
 
 	gui.Views.Confirmation.ScrollUp(gui.Views.Confirmation.ViewLinesHeight())
@@ -141,7 +141,7 @@ func (gui *Gui) goToConfirmationPanelTop() error {
 
 func (gui *Gui) goToConfirmationPanelBottom() error {
 	if gui.Views.Confirmation.Editable {
-		return nil
+		return gocui.ErrKeybindingNotHandled
 	}
 
 	gui.Views.Confirmation.ScrollDown(gui.Views.Confirmation.ViewLinesHeight())
